@@ -3,6 +3,7 @@ import yaml
 from serverless.aws.functions.event_bridge import EventBridgeFunction
 from serverless.aws.functions.generic import Function
 from serverless.aws.functions.http import HTTPFunction
+from serverless.aws.functions.s3 import S3Function
 from serverless.aws.iam import IAMManager
 from serverless.service.environment import Environment
 from serverless.service.types import Provider as BaseProvider
@@ -145,12 +146,18 @@ class FunctionBuilder:
 
         return fn
 
+    def s3(self, name, description, bucket, event, rules=None, handler=None, timeout=None, layers=None, **kwargs):
+        fn = S3Function(self.service, name, description, bucket, event, rules, handler, timeout, layers, **kwargs)
+        self.service.functions.add(fn)
+
+        return fn
+
 
 class Provider(BaseProvider, yaml.YAMLObject):
     yaml_tag = "!Provider"
 
     def __init__(
-        self, runtime=Runtime.PYTHON_3_8, extra_tags=None, timeout=10, stage="${opt:stage}", environment=None, **kwargs
+        self, runtime=Runtime.PYTHON_3_8, extra_tags=None, timeout=10, stage="${sls:stage}", environment=None, **kwargs
     ):
         super().__init__(**kwargs)
         self.deploymentBucket = None
@@ -159,7 +166,7 @@ class Provider(BaseProvider, yaml.YAMLObject):
 
         self.name = "aws"
         self.runtime = runtime
-        self.stackName = "${self:service}"
+        self.stackName = "${self:service}-${sls:stage}"
         self.timeout = timeout
         self.stage = stage
         self.tags = extra_tags or {}
@@ -181,7 +188,7 @@ class Provider(BaseProvider, yaml.YAMLObject):
     def to_yaml(cls, dumper, data):
         data.pop("_service", None)
         data.pop("function_builder", None)
-        if not data["deploymentBucket"]:
+        if not data.get("deploymentBucket"):
             data.pop("deploymentBucket", None)
 
         return dumper.represent_dict(data)
